#!/usr/bin/env python2.4
# -*- coding: utf8 -*-

# wxConvert a little tool for O.r.i.g.i.n.a.l.
#
# by Yoan Blanc <yoan.at.dosimple.ch>
#
# released under the GPL license.

import datetime
import Image
import os
import re
import sys
import wx
from zipfile import ZipFile
from ftplib import FTP
from ConfigParser import ConfigParser

__author__ = "Yoan BLANC, dosimple.ch"
__version__ = "0.2"
__license__ = "GPL"
__revision__ = "20051009"

configuration = "config.ini"

# Read the configuration from a file
if os.path.exists(configuration):
	cp = ConfigParser()
	cp.read([configuration])
else:
	raise Exception, "Configuration file (%s) is missing." % (configuration)

# general
try:
	zip = eval(cp.get('general', 'zip'))
	comments = eval(cp.get('general', 'comments'))
	generated = eval(cp.get('general', 'generated'))
	supportedFiles = eval(cp.get('general', 'supportedFiles'))
	zipped = eval(cp.get('general', 'zipped'))
except Exception:
	raise Exception, "Error in configuration file, section [general]"

# encoding
try:
	decoding = cp.get('encoding', 'input')
except Exception:
	decoding = False
	pass

try:
	encoding = cp.get('encoding', 'output')
except Exception:
	encoding = "utf-8"

# internal
try:
	rootDir = cp.get('internal', 'rootDir')
	imgName = cp.get('internal', 'imgName')
	infoName = cp.get('internal', 'infoName')
except Exception:
	raise Exception, "Error in configuration file, section [internal]"

# ftp
ftp = {}
for i in cp.items('ftp'):
	ftp[i[0]] = i[1]
# conf
conf = {}
for i in cp.items('conf'):
	conf[i[0]] = eval(i[1])

dirs = {
	'thumbs':rootDir + os.sep + 'thumbs' + os.sep,
	'lq':rootDir + os.sep + 'lq' + os.sep,
	'mq':rootDir + os.sep + 'mq' + os.sep,
	'hq':rootDir + os.sep + 'hq' + os.sep,
	'zip':rootDir + os.sep + 'zip' + os.sep,
	'comments':rootDir + os.sep + 'comments' + os.sep
}

#------------------------------------------------------------------------------

class Gallery:
	"""class that generates the web-gallery for the given files"""
	def __init__(self, rootpath, fileslist, infos={}):
		self.rootpath = rootpath
		self.fileslist = fileslist
		
		os.chdir(self.rootpath)
		os.mkdir(rootDir)
		for g in generated:
			os.mkdir(dirs[g])
		if zip : os.mkdir(dirs['zip'])
		os.mkdir(dirs['comments'])
		if comments: os.chmod(dirs['comments'], 0777)
		
		# writes the gallery informations
		finfo = open(rootDir + os.sep + infoName, 'w')
		for k, v in infos.items():
			if(len(v) > 0):
				s = "%s|%s\n" % (k, v)
				if decoding:
					s = s.decode(decoding)
				finfo.write(s.encode(encoding, "xmlcharrefreplace"))
		finfo.close()
	
	def Generate(self):
		"""generate the thumbs, lq and mq pics"""
		os.chdir(self.rootpath)
		
		i = 1
		
		for f in self.fileslist:
			for g in generated:
				im = Image.open(f[0])
				if conf[g].has_key('crop') and conf[g]['crop']:
					# paysage
					if im.size[0] > im.size[1]:
						im = im.crop(((im.size[0] - im.size[1]) / 2, 0, \
							(im.size[0] + im.size[1]) / 2, im.size[1]))
					# portrait
					else:
						im = im.crop((0, (im.size[1] - im.size[0]) / 2, \
							im.size[0], (im.size[1] + im.size[0]) / 2))
				im.thumbnail((conf[g]['geom']), Image.ANTIALIAS)
				im.save("%s%s%d.jpg" % (dirs[g], imgName, i), \
					"JPEG", quality=conf[g]['quality'])
			
			c = open("%s%d.txt" % (dirs['comments'], i), "w")
			s = "<span>%s</span>\n" % (f[1])
			if decoding:
				s = s.decode(decoding)
			c.write(s.encode(encoding, "xmlcharrefreplace"))
			c.close()
			
			yield i
			i += 1
	
	def Zip(self):
		os.chdir(self.rootpath)
		os.mkdir(dirs['zip'])
		
		for z in zipped:
			zip = ZipFile("%s%s.zip" % (dirs['zip'], z), "w")
			for i in range(1,len(self.fileslist)):
				zip.write(("%s%s%d.jpg" % (dirs[z], imgName, i)))
			zip.close()
		
#------------------------------------------------------------------------------

ID_OPEN = 10
ID_ABOUT = 11
ID_BUTTON = 50
ID_LIST = 89
ID_EXIT = 90

_dirRegex = re.compile(r'\d{2}:\d{2} (\w+)$')
_titleRegex = re.compile(r'^\w+$')
_dateRegex = re.compile(r'^[0-3][0-9]\.[0-1][0-9]\.(19|20)[0-9]{2}$')

class wxConvert(wx.App):	
	def OnInit(self):
		"""Inits the app"""
		self.frame = wx.Frame(None, -1, \
			"wxConvert " + __version__, size=(640, 480), \
			style=wx.DEFAULT_FRAME_STYLE \
			|wx.NO_FULL_REPAINT_ON_RESIZE \
			|wx.TAB_TRAVERSAL)
		
		self.frame.CreateStatusBar()
		
		panel = MainPanel(self.frame, -1)
		
		self.frame.SetMenuBar(self.__buildMenu())
		
		#wx.EVT_MENU(self, ID_OPEN, self.OnOpen)
		wx.EVT_MENU(self.frame, ID_ABOUT, self.OnAbout)
		wx.EVT_MENU(self.frame, ID_EXIT, self.Exit)
		
		self.SetTopWindow(self.frame)
		self.frame.Show(True)
		return True
	
	def __buildMenu(self):
		"""inits the menu"""
		filemenu = wx.Menu()
		#filemenu.Append(ID_OPEN, "&Open", "Open a directory")
		#filemenu.AppendSeparator()
		filemenu.Append(ID_ABOUT, "&About", "Information about wxConvert")
		filemenu.AppendSeparator()
		filemenu.Append(ID_EXIT, "E&xit", "Quit")
		
		menubar = wx.MenuBar()
		menubar.Append(filemenu, "&File")
		
		return menubar

	def OnAbout(self, e):
		"""act with the about event"""
		dialog = wx.MessageDialog(self.frame, "wxConvert " + __version__ +"\n\n"
			"for Jimmac's O.r.i.g.i.n.a.l.\n"
			"by " + __author__
			, "About wxConvert", \
			wx.OK)
		
		dialog.ShowModal()
		dialog.Destroy()
	
	def Exit(self, e = None):
		"""act with the exit event"""
		dialog = wx.MessageDialog(self.frame, "Do you really wanna quit ?", \
			"Quit", wx.YES_NO|wx.NO_DEFAULT)
		
		close = dialog.ShowModal() == wx.ID_YES
		dialog.Destroy()
		
		if close:
			wx.App.Exit(self)

class ImgPanel(wx.Panel):
	def __init__(self, parent, id):
		wx.Panel.__init__(self, parent, id, style=wx.FULL_REPAINT_ON_RESIZE)
		self.image = None
		self.pos = (0, 0)
		wx.EVT_PAINT(self, self.OnPaint)
	
	def SetImage(self, filename):
		im = Image.open(filename)
		im.thumbnail(self.GetSize(), Image.ANTIALIAS)
		self.image = wx.EmptyImage(im.size[0], im.size[1])
		self.image.SetData(im.convert('RGB').tostring())
		self.pos = ((self.GetSize()[0] - im.size[0]) / 2, \
			(self.GetSize()[1] - im.size[1]) / 2)
		self.Refresh()
		self.Update()
	
	def Clear(self):
		self.image = None
		self.Refresh()
		self.Update()
	
	def OnPaint(self, e):
		if self.image:
			dc = wx.PaintDC(self)
			dc.DrawBitmap(self.image.ConvertToBitmap(), self.pos[0], \
				self.pos[1])

class MainPanel(wx.Panel):
	"""Main panel"""
	def __init__(self, parent, id=-1):
		
		wx.Panel.__init__(self, parent, id, wx.DefaultPosition, \
			wx.DefaultSize, wx.TAB_TRAVERSAL, "MainPanel")
		
		self.frame = parent;
		self.dirname = '.'
		self.fileslist = []
		self.installedGaleries = []
		self.selectedImage = -1
		
		self.__initLayout()
		self.__open()
		
		# Events
		wx.EVT_BUTTON(self, ID_OPEN, self.OnOpen)
		wx.EVT_BUTTON(self, ID_BUTTON, self.OnGenerate)
		wx.EVT_BUTTON(self, ID_BUTTON+1, self.OnUpload)
		wx.EVT_BUTTON(self, ID_BUTTON+2, self.OnApply)
		wx.EVT_BUTTON(self, ID_BUTTON+3, self.OnRemove)
		
		wx.EVT_LIST_ITEM_SELECTED(self, ID_LIST, self.OnItemSelected)
		
		self.Centre(wx.HORIZONTAL | wx.VERTICAL)
		self.Show(True)
	
	def __initLayout(self):
		"""Inits the layout"""
		self.buttons = {}
		self.fields = {}
		
		vbox = wx.BoxSizer(wx.VERTICAL)
		vsplit = wx.BoxSizer(wx.HORIZONTAL)
		hsplit = wx.BoxSizer(wx.VERTICAL)
		
		# Path
		label = wx.StaticText(self, wx.NewId(), "Path : ")
		self.dirtext = wx.TextCtrl(self, -1, self.dirname)
		self.dirtext.SetEditable(False)
		self.buttons['open'] = wx.Button(self, ID_OPEN, "Open")
		
		selbox = wx.BoxSizer(wx.HORIZONTAL)
		
		selbox.Add(label, 0, wx.ALIGN_CENTER_VERTICAL)
		selbox.Add(self.dirtext, 1, wx.EXPAND)
		selbox.Add(self.buttons['open'])
		
		# ImageList
		self.list = wx.ListView(self, ID_LIST)
		#self.list.InsertColumn(0, "preview")
		self.list.InsertColumn(0, "Filename")
		self.list.InsertColumn(1, "Title")
		
		# Informations
		infos = wx.FlexGridSizer(4, 2, 2, 2)
		infos.AddGrowableCol(1,1)
		infos.SetFlexibleDirection(wx.HORIZONTAL)
		infos.SetNonFlexibleGrowMode(wx.FLEX_GROWMODE_SPECIFIED)
		
		infoslabel = wx.StaticBox(self, label="Gallery informations")
		infosbox = wx.StaticBoxSizer(infoslabel)
		infosbox.Add(infos, 1, wx.EXPAND)
		
		fields = (
			# name, label, initial value, max length
			('name','Name : ',''),
			('description', 'Description : ',''),
			('author', 'Author : ',''),
			('date', 'Date : ', \
				datetime.date.strftime(datetime.date.today(), "%d.%m.%Y"), 10)
		)
		
		l = wx.StaticText(self, wx.ID_ANY, "Title : ")
		self.title = wx.TextCtrl(self, wx.ID_ANY, os.path.basename(self.dirname))
		infos.Add(l)
		infos.Add(self.title, 1, wx.EXPAND)
		
		for field in fields:
			l = wx.StaticText(self, wx.ID_ANY, field[1])
			t = wx.TextCtrl(self, wx.ID_ANY, field[2])
			if len(field) > 3:
				t.SetMaxLength(field[3])
			infos.Add(l)
			infos.Add(t, 1, wx.EXPAND)
			self.fields[field[0]] = t
		
		# Image editor
		
		imgedit = wx.BoxSizer(wx.VERTICAL)
		imgt = wx.BoxSizer(wx.HORIZONTAL)
		
		self.imgpanel = ImgPanel(self, -1)
		
		imgtl = wx.StaticText(self, wx.ID_ANY, "Title : ")
		self.imgtitle = wx.TextCtrl(self, wx.ID_ANY, "")
		imgapply = wx.Button(self, ID_BUTTON+2, "Apply")
		imgremove = wx.Button(self, ID_BUTTON+3, "Remove")
		
		imgapply.SetDefault()
		
		b=5
		
		imgt.Add(imgtl, 0, wx.ALIGN_CENTER_VERTICAL)
		imgt.Add(self.imgtitle, 1, wx.EXPAND)
		imgt.Add(imgapply, flag=wx.LEFT, border=b)
		imgt.Add(imgremove, flag=wx.LEFT, border=b)
		
		imgedit.Add(self.imgpanel, 1, wx.EXPAND)
		imgedit.Add(imgt, 0, wx.ALL|wx.EXPAND,border=b)
		
		# Buttons
		
		hbox = wx.BoxSizer(wx.HORIZONTAL)
		
		self.buttons['gen'] = wx.Button(self, ID_BUTTON, "Generate")
		self.buttons['upload'] = wx.Button(self, ID_BUTTON+1, "Upload")
		
		hbox.Add(self.buttons['gen'])
		hbox.Add(wx.Size(20,1))#, 1, wx.EXPAND)
		hbox.Add(self.buttons['upload'])
		
		# End
		
		# border
		b = 5
		
		hsplit.Add(infosbox, 0, wx.ALL|wx.EXPAND, border=b)
		hsplit.Add(imgedit, 1, wx.EXPAND)
		
		vsplit.Add(self.list, 1, wx.ALL|wx.EXPAND, border=b)
		vsplit.Add(hsplit, 1, wx.EXPAND)
		
		vbox.Add(selbox, 0, wx.ALL|wx.EXPAND, border=b)
		vbox.Add(vsplit, 1, wx.EXPAND)
		vbox.Add(hbox, 0, wx.ALL|wx.EXPAND, border=b)
		
		self.SetSizer(vbox)
		vbox.Fit(self)
		#self.AddChild(hsplit)
	
	def __open(self):
		"""Opens the directory"""
		fileslist = [((os.path.splitext(i)[1].lower() in supportedFiles) \
			and [i] or [None])[0] \
			for i in os.listdir(self.dirname)]
		
		for c in xrange(fileslist.count(None)):
			fileslist.remove(None)
		
		i = 0
		for f in fileslist:
			self.fileslist.append([f, "image %d" % (i+1,)])
			i+=1
		
		if os.path.exists(self.dirname + os.sep +  rootDir):
			"""Reading an existing configuration"""
			f = open(self.dirname + os.sep + rootDir + os.sep + infoName)
			for l in f.readlines():
				v = l.decode(encoding).split('|',1)
				if len(v) == 2 and self.fields.has_key(v[0]):
					self.fields[v[0]].SetValue(v[1][:-1])
				
		self.__fillList()
	
	def __fillList(self):
		"""Fills the list with the list of pictures"""
		self.list.DeleteAllItems()
		
		for f in self.fileslist:
			self.list.Append(f)
	
	def __upload(self):
		"""Uploads the gallery"""
		try:
			f = FTP(ftp['host'])
			f.login(ftp['user'], ftp['password'])
			f.cwd(ftp['basedir'])
			
			self.__installedGaleries(f)
			
			galName = os.path.basename(self.title.GetValue())
			
			if galName in self.installedGaleries :
				error = wx.MessageDialog(self, "The gallery called '" + galName + \
					"' already exists on the server.\n\n"
					"Renaming the title of the gallery could fix this problem.", \
					"Upload error", wx.OK|wx.CENTER)
				error.ShowModal()
				error.Destroy()
			else:
				f.mkd(galName)
				f.cwd(galName)
				
				progress = wx.ProgressDialog("Upload",
					"Uploading the gallery")
				size = recCount(self.dirname + os.sep + rootDir)
				
				for i in self.__uploadFiles(self.dirname + os.sep + rootDir, f):
					msg = "Image %s of %s" % (i, size)
					self.frame.GetStatusBar().SetStatusText(msg)
					progress.Update(i*100/size, msg)
					wx.Yield()
				
				self.frame.GetStatusBar().SetStatusText("");
				progress.Destroy()
				
				if comments:
					f.sendcmd("SITE CHMOD 0777 comments")
		except Exception, e:
			dialog = wx.MessageDialog(self, str(e), "FTP Error", \
				wx.OK|wx.CENTER)
			dialog.ShowModal()
			dialog.Destroy()
		else:
			f.close()
			return
	
	def __installedGaleries(self, ftpcon):
		"""Gets the already installed galleries (on the FTP)"""
		self.installedGaleries = []
		ftpcon.retrlines('LIST -D', self.__filterDir)
	
	def __filterDir(self, line):
		"""Callback function for FTP"""
		m = _dirRegex.search(line)
		if m is not None:
			self.installedGaleries.append(m.group(1))
	
	def __uploadFiles(self, dir, ftpcon):
		"""Uploads the content of web-gallery"""
		i = 1
		for file in os.listdir(dir):
			filename = os.path.join(dir, file)
			if os.path.isdir(filename):
				ftpcon.mkd(file)
				for f in os.listdir(os.path.join(dir, filename)):
					if i % 10 == 9 : ftpcon.cwd('.') # reactive the ftp timer
					fname = os.path.join(dir, filename, f)
					if os.path.splitext(f)[1] == '.txt':
						fp = open(fname, 'r')
						ftpcon.storlines("STOR " + file + '/' + f, fp)
					else:
						fp = open(fname, 'rb')
						ftpcon.storbinary("STOR " + file + '/' + f, fp)
					yield i
					i += 1
			else:
				fp = open(filename, 'r')
				ftpcon.storlines("STOR " + file, fp)
				yield i
				i += 1
	
	def __validation(self):
		"""Validate the fields of title and date"""
		if _dateRegex.match(self.fields['date'].GetValue()) == None:
			return "Date doesn't match with dd.mm.yyyy"
		elif _titleRegex.match(self.title.GetValue()) == None:
			return "Title doesn't match, only alphanumerical characters are allowed."
		else:
			return None
	
	def OnApply(self, e):
		"""Applies the title to the picture"""
		if(self.selectedImage >= 0):
			title = self.imgtitle.GetValue()
			self.fileslist[self.selectedImage][1] = title
			
			self.__fillList()
	
	def OnGenerate(self, e):
		"""Acts with the generate event"""		
		errorMsg = self.__validation()
		
		if errorMsg:
			dialog = wx.MessageDialog(self, errorMsg, "Error", wx.OK|wx.CENTER)
			dialog.ShowModal()
			dialog.Destroy()
			return
		
		gen = True
		
		if os.path.exists(os.path.join(self.dirname, rootDir)):
			dialog = wx.MessageDialog(self, "The directory '" + rootDir + \
				"' already exists in '" + self.dirname +"'.\n\n"
				"Would you overwrite it ?", "Override existing directory", \
				wx.YES_NO|wx.NO_DEFAULT|wx.CENTER)
			
			if dialog.ShowModal() == wx.ID_YES:
				gen = True
				rmrf(os.path.join(self.dirname, rootDir))
			else:
				gen = False
			
			dialog.Destroy()
		
		if gen:
			infos = {}
			for k,v in self.fields.items():
				infos[k] = v.GetValue()
			
			gallery = Gallery(self.dirname, self.fileslist, infos)
			
			progress = wx.ProgressDialog("Generation",
				"Generating the gallery")
			size = len(self.fileslist)
			
			for i in gallery.Generate():
				msg = "Image %d of %d" % (i+1, size)
				self.frame.GetStatusBar().SetStatusText(msg)
				progress.Update(i*100/size, msg)
				wx.Yield()
			
			self.frame.GetStatusBar().SetStatusText("");
			progress.Destroy()
			
			if zip:
				gallery.Zip()
	
	def OnItemSelected(self, e):
		self.selectedImage = e.m_itemIndex;
		img = self.fileslist[self.selectedImage]
		self.imgtitle.SetValue(img[1])
		self.imgpanel.SetImage(self.dirname + os.sep +img[0])
	
	def OnOpen(self, e):
		"""Acts with the open event"""
		dirdialog = wx.DirDialog(self, "Choose a directory", self.dirname, \
			wx.DD_NEW_DIR_BUTTON)
		
		if dirdialog.ShowModal() == wx.ID_OK:
			self.dirname = dirdialog.GetPath()
			self.dirtext.SetValue(self.dirname);
			self.title.SetValue(os.path.basename(self.dirname))
			
			self.__open()
		
		dirdialog.Destroy()
	
	def OnRemove(self, e):
		"""Acts when a picture is removed from the gallery"""
		gen = True
		if os.path.exists(os.path.join(self.dirname, rootDir)):
			dialog = wx.MessageDialog(self, "Would you remove this picture "
				"from the gallery ? (only remove from the list not physically",\
				"Remove " + self.fileslist[self.selectedImage][1] + " ?", \
				wx.YES_NO|wx.NO_DEFAULT|wx.CENTER)
			
			if dialog.ShowModal() == wx.ID_YES:
				self.fileslist.__delitem__(self.selectedImage)
				for i in range(self.selectedImage, len(self.fileslist)):
					if(self.fileslist[i][1] == ("image %d" % (i+2,))):
						self.fileslist[i][1] = ("image %d" % (i+1,))
					else:
						print self.fileslist[i][1] + (" :: image %d" % (i+2,))
				self.__fillList()
				select = self.selectedImage
				if select < len(self.fileslist):
					self.list.Select(select)
				elif len(self.fileslist) == 0:
					self.imgtitle.SetValue("")
					self.imgpanel.Clear()
					self.selectedImage = -1
				else:
					self.list.Select(select-1)
			
			dialog.Destroy()
	
	def OnUpload(self, e):
		"""Uploads the files to the server"""
		if not os.path.exists(os.path.join(self.dirname, rootDir)):
			dialog = wx.MessageDialog(self, "The gallery isn't yet "
				"generated.", "No generated gallery", \
				wx.OK)
			
			gen = dialog.ShowModal() == wx.ID_YES
			dialog.Destroy()
		else:
			self.__upload()

#------------------------------------------------------------------------------

def recCount(dir):
	i = 0
	for root, dirs, files in os.walk(dir, topdown=False):
		i += len(files)
	return i

def rmrf(dir):
	"""rm -rf"""
	for root, dirs, files in os.walk(dir, topdown=False):
		for name in files:
			os.remove(os.path.join(root, name))
		for name in dirs:
			os.rmdir(os.path.join(root, name))
	os.rmdir(dir)

def main(argv):	
	#app = ((len(argv) > 1) and [wxConvert(argv[1])] or [wxConvert()])[0]
	app = wxConvert()
	app.MainLoop()

if __name__ == "__main__" : main(sys.argv)